@@ -53,6 +53,7 @@
 #include <QStackedLayout>
 #include <QLabel>
 #include <QPushButton>
+#include <QInputDialog>
 
 ProjectTab::ProjectTab(QWidget *parent)
     : LokalizeSubwindowBase2(parent)
@@ -268,6 +269,7 @@ void ProjectTab::contextMenuEvent(QContextMenuEvent *event)
     menu->addAction(i18nc("@action:inmenu", "Add to translation memory"), this, &ProjectTab::scanFilesToTM);
 
     menu->addAction(i18nc("@action:inmenu", "Search in files"), this, &ProjectTab::searchInFiles);
+    menu->addAction(i18nc("@action:inmenu", "Add a comment"), this, &ProjectTab::addComment);
     if (Settings::self()->pologyEnabled()) {
         menu->addAction(i18nc("@action:inmenu", "Launch Pology on files"), this, &ProjectTab::pologyOnFiles);
     }
@@ -281,7 +283,6 @@ void ProjectTab::contextMenuEvent(QContextMenuEvent *event)
 //         menu.addAction(i18n("Force Scanning"),this,&ProjectTab::slotForceStats);
 //
 //     }
-
     menu->popup(event->globalPos());
 }
 
@@ -291,6 +292,41 @@ void ProjectTab::scanFilesToTM()
     TM::scanRecursive(m_browser->selectedItems(), Project::instance()->projectID());
 }
 
+void ProjectTab::addComment()
+{
+    QStringList files = m_browser->selectedItems();
+    int i = files.size();
+    QStringList previousCommentsTexts = Project::instance()->commentsTexts();
+    QStringList previousCommentsFiles = Project::instance()->commentsFiles();
+    QString previousComment(QStringLiteral(""));
+    if (i >= 1) {
+        //Retrieve previous comment (first one)
+        int existingItem = previousCommentsFiles.indexOf(Project::instance()->relativePath(files.at(0)));
+        if (existingItem != -1 && previousCommentsTexts.count() > existingItem) {
+            previousComment = previousCommentsTexts.at(existingItem);
+        }
+    }
+
+    bool ok;
+    QString newComment = QInputDialog::getText(this, i18n("Project file comment"), i18n("Input a comment for this project file:"), QLineEdit::Normal, previousComment, &ok);
+    if (!ok)
+        return;
+
+    while (--i >= 0) {
+        QString filePath = Project::instance()->relativePath(files.at(i));
+        int existingItem = previousCommentsFiles.indexOf(filePath);
+        if (existingItem != -1 && previousCommentsTexts.count() > existingItem) {
+            previousCommentsTexts[existingItem] = newComment;
+        } else {
+            previousCommentsTexts << newComment;
+            previousCommentsFiles << filePath;
+        }
+    }
+    Project::instance()->setCommentsTexts(previousCommentsTexts);
+    Project::instance()->setCommentsFiles(previousCommentsFiles);
+    Project::instance()->save();
+}
+
 void ProjectTab::searchInFiles(bool templ)
 {
     QStringList files = m_browser->selectedItems();
@@ -362,12 +398,12 @@ void ProjectTab::openFile()
                        + QStringLiteral("\n\n")
                        + i18n("Are you sure you want to open this many files?");
         auto yes = KGuiItem(
-            i18np("&Open %1 File", "&Open %1 Files", i),
-            QStringLiteral("document-open")
-        );
+                       i18np("&Open %1 File", "&Open %1 Files", i),
+                       QStringLiteral("document-open")
+                   );
         const int answer = KMessageBox::warningYesNo(
-            this, text, caption, yes, KStandardGuiItem::cancel()
-        );
+                               this, text, caption, yes, KStandardGuiItem::cancel()
+                           );
         if (answer != KMessageBox::Yes) {
             return;
         }
@@ -375,7 +411,10 @@ void ProjectTab::openFile()
 
     while (--i >= 0) {
         if (Catalog::extIsSupported(files.at(i)))  {
-            emit fileOpenRequested(files.at(i), true);
+            if (files.at(i).endsWith(QLatin1String(".pot")))
+                emit fileOpenRequested(Project::instance()->model()->potToPo(files.at(i)).toLocalFile(), true);
+            else
+                emit fileOpenRequested(files.at(i), true);
         }
     }
 }
